<?php

namespace Intergo\AuthClient\Rules;

use Illuminate\Contracts\Validation\Rule;
use Illuminate\Support\Str;

class PasswordRule implements Rule
{
    const MIN_LENGTH = 15;
    const MAX_LENGTH = 32;

    const MESSAGE = 'Password needs to have at least one uppercase, one lowercase, one special character and one digit.';

    private bool $minLengthPasses = true;
    private bool $maxLengthPasses = true;
    /**
     * Determine if the validation rule passes.
     *
     * @param string $attribute
     * @param mixed $value
     * @return bool
     */
    public function passes($attribute, $value)
    {
        //somehow it passes array and this is related with task https://app.asana.com/0/1202041647164570/1202140521843719/f
        if (!is_string($value)) {
            return false;
        }

        if (Str::length($value) < self::MIN_LENGTH) {
            $this->minLengthPasses = false;
            return false;
        }

        if (Str::length($value) > self::MAX_LENGTH) {
            $this->maxLengthPasses = false;
            return false;
        }

        /**
         * Password requires at least:
         *  1 digit
         *  1 uppercase letter
         *  1 lowercase letter
         *  1 special character
         */
        return preg_match('/[0-9]/', $value)
            && preg_match('/[A-Z]/', $value)
            && preg_match('/[a-z]/', $value)
            && preg_match('/[^A-Za-z0-9]/', $value);
    }

    /**
     * Get the validation error message.
     *
     * @return string
     */
    public function message()
    {
        if (!$this->minLengthPasses) {
            return sprintf('The :attribute must be at least %d characters.', self::MIN_LENGTH);
        }

        if (!$this->maxLengthPasses) {
            return sprintf('The :attribute may not be greater than %d characters.', self::MAX_LENGTH);
        }

        return self::MESSAGE;
    }
}
